@@ -1,4 +1,4 @@
-# (C) British Crown Copyright 2010 - 2018, Met Office
+# (C) British Crown Copyright 2010 - 2019, Met Office
 #
 # This file is part of Iris.
 #
@@ -1241,9 +1241,15 @@ def _proportion(array, function, axis, **kwargs):
     # values
     if ma.isMaskedArray(array):
         # calculate the total number of non-masked values across the given axis
-        total_non_masked = count(
-            array.mask, axis=axis, function=np.logical_not, **kwargs)
-        total_non_masked = ma.masked_equal(total_non_masked, 0)
+        if array.mask is np.bool_(False):
+            # numpy will return a single boolean as a mask if the mask
+            # was not explicitly specified on array construction, so in this
+            # case pass the array shape instead of the mask:
+            total_non_masked = array.shape[axis]
+        else:
+            total_non_masked = count(
+                array.mask, axis=axis, function=np.logical_not, **kwargs)
+            total_non_masked = ma.masked_equal(total_non_masked, 0)
     else:
         total_non_masked = array.shape[axis]
 
@@ -1318,7 +1324,9 @@ def _peak(array, **kwargs):
 
     # Collapse array to its final data shape.
     slices = [slice(None)] * array.ndim
-    slices[-1] = 0
+    endslice = slice(0, 1) if len(slices) == 1 else 0
+    slices[-1] = endslice
+    slices = tuple(slices)  # Numpy>=1.16 : index with tuple, *not* list.
 
     if isinstance(array.dtype, np.float):
         data = array[slices]
