@@ -1,4 +1,4 @@
-# (C) British Crown Copyright 2010 - 2018, Met Office
+# (C) British Crown Copyright 2010 - 2019, Met Office
 #
 # This file is part of Iris.
 #
@@ -331,19 +331,42 @@ def rolling_window(a, window=1, step=1, axis=-1):
     return rw
 
 
-def array_equal(array1, array2):
+def array_equal(array1, array2, withnans=False):
     """
     Returns whether two arrays have the same shape and elements.
 
-    This provides the same functionality as :func:`numpy.array_equal` but with
-    additional support for arrays of strings.
+    Args:
+
+    * array1, array2 (arraylike):
+        args to be compared, after normalising with :func:`np.asarray`.
+
+    Kwargs:
+
+    * withnans (bool):
+        When unset (default), the result is False if either input contains NaN
+        points.  This is the normal floating-point arithmetic result.
+        When set, return True if inputs contain the same value in all elements,
+        _including_ any NaN values.
+
+    This provides much the same functionality as :func:`numpy.array_equal`, but
+    with additional support for arrays of strings and NaN-tolerant operation.
 
     """
     array1, array2 = np.asarray(array1), np.asarray(array2)
-    if array1.shape != array2.shape:
-        eq = False
-    else:
-        eq = bool(np.asarray(array1 == array2).all())
+
+    eq = (array1.shape == array2.shape)
+    if eq:
+        eqs = (array1 == array2)
+
+        if withnans and (array1.dtype.kind == 'f' or array2.dtype.kind == 'f'):
+            nans1, nans2 = np.isnan(array1), np.isnan(array2)
+            if not np.all(nans1 == nans2):
+                eq = False  # simply fail
+            else:
+                eqs[nans1] = True  # fix NaNs; check all the others
+
+        if eq:
+            eq = np.all(eqs)  # check equal at all points
 
     return eq
 
@@ -1061,8 +1084,12 @@ def new_axis(src_cube, scalar_coord=None):
         coord_dims = np.array(src_cube.coord_dims(coord)) + 1
         new_cube.add_dim_coord(coord.copy(), coord_dims)
 
+    nonderived_coords = src_cube.dim_coords + src_cube.aux_coords
+    coord_mapping = {id(old_co): new_cube.coord(old_co)
+                     for old_co in nonderived_coords}
     for factory in src_cube.aux_factories:
-        new_cube.add_aux_factory(copy.deepcopy(factory))
+        new_factory = factory.updated(coord_mapping)
+        new_cube.add_aux_factory(new_factory)
 
     return new_cube
 
